@@ -11,7 +11,9 @@ import json
 
 import ecmwfapi
 
+from climetlab.core.thread import SoftThreadPool
 from climetlab.normalize import normalize_args
+from climetlab.utils import tqdm
 
 from .file import FileSource
 from .prompt import APIKeyPrompt
@@ -77,12 +79,28 @@ def service(name):
 
 class MARSRetriever(FileSource):
     def __init__(self, **kwargs):
-        request = self.request(**kwargs)
 
+        requests = self.requests(**kwargs)
+
+        service("mars")  # Trigger password prompt before thraeding
+
+        nthreads = min(self.settings("number-of-download-threads"), len(requests))
+
+        if nthreads < 2:
+            self.path = [self._retrieve(r) for r in requests]
+        else:
+            with SoftThreadPool(nthreads=nthreads) as pool:
+
+                futures = [pool.submit(self._retrieve, r) for r in requests]
+
+                iterator = (f.result() for f in futures)
+                self.path = list(tqdm(iterator, leave=True, total=len(requests)))
+
+    def _retrieve(self, request):
         def retrieve(target, request):
             service("mars").execute(request, target)
 
-        self.path = self.cache_file(
+        return self.cache_file(
             retrieve,
             request,
         )
@@ -92,8 +110,19 @@ class MARSRetriever(FileSource):
         date="date-list(%Y-%m-%d)",
         area="bounding-box(list)",
     )
-    def request(self, **kwargs):
-        return kwargs
+    def requests(self, **kwargs):
+        split_on = kwargs.pop("split_on", None)
+        if split_on is None or not isinstance(kwargs.get(split_on), (list, tuple)):
+            return [kwargs]
+
+        result = []
+
+        for v in kwargs[split_on]:
+            r = dict(**kwargs)
+            r[split_on] = v
+            result.append(r)
+
+        return result
 
     def to_pandas(self, **kwargs):
 
